/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2003 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>	/* umask() */
#include <sys/stat.h>	/* umask() */
#include <stdio.h>		/* puts() */
#include <stdlib.h>		/* putenv() */
#include <string.h>		/* strcmp() */
#include <unistd.h>		/* getpid() */

#include "clex.h"

#include "cfg.h"		/* config_initialize() */
#include "completion.h"	/* completion_initialize() */
#include "control.h"	/* control_loop() */
#include "directory.h"	/* dir_initialize() */
#include "exec.h"		/* exec_initialize() */
#include "filepanel.h"	/* files_initialize() */
#include "help.h"		/* help_initialize() */
#include "history.h"	/* hist_initialize() */
#include "inout.h"		/* curses_initialize() */
#include "list.h"		/* list_initialize() */
#include "select.h"		/* select_initialize() */
#include "signals.h"	/* signal_initialize() */
#include "tty.h"		/* tty_initialize() */
#include "userdata.h"	/* userdata_initialize() */
#include "ustring.h"	/* US_INIT() */
#include "util.h"		/* base_name() */
#include "xterm_title.h"/* xterm_title_initialize() */

/* GLOBAL VARIABLES */
PANEL *panel = 0;
/*
 * initialization:
 *   cnt, top, curs, min, type, norev, infoline
 *   <panel specific data>
 */
PANEL_COMPL panel_compl =
	{  0,0,0,-1,PANEL_TYPE_COMPL,		0,1 };
PANEL_CFG panel_cfg =
	{  0,0,0,-1,PANEL_TYPE_CFG,			0,1 };
PANEL_CFG_CONFIRM panel_cfg_confirm =
	{  0,0,0, 0,PANEL_TYPE_CFG_CONFIRM,	0,0 };
PANEL_DIR panel_dir =
	{  0,0,0,-1,PANEL_TYPE_DIR,			0,0 };
PANEL_DIR_SPLIT panel_dir_split =
	{  0,0,0,-1,PANEL_TYPE_DIR_SPLIT,	0,0 };
PANEL_HELP panel_help =
	{  0,0,0, 0,PANEL_TYPE_HELP,		1,0 };
PANEL_HIST panel_hist =
	{  0,0,0,-1,PANEL_TYPE_HIST,		0,0 };
PANEL_LOCATE panel_locate =
	{  0,0,0, 0,PANEL_TYPE_LOCATE,		0,1,
	  0,0 };
PANEL_MAINMENU panel_mainmenu =
	/* 19 corresponding functions in control.c */
	{ 19,-1,-1,-1,PANEL_TYPE_MAINMENU,	0,0 };
PANEL_SORT panel_sort =
	/* 7 SORT_XXXs defined in clex.h */ 
	{  7,0,0,-1,PANEL_TYPE_SORT,		0,0,
	  SORT_NAME  };
PANEL_FILE *ppanel_file;
DISPLAY display = { 0,0,0,0,0,0,0 };
CLEX_DATA clex_data;
TEXTLINE *textline = 0, line_cmd, line_tmp, line_dir;
CODE next_mode;

int
main(int argc, char *argv[])
{
	int i;
	FLAG help, version;
	const char *arg;

	/* check command line arguments */
	clex_data.admin = help = version = 0;
	for (i = 1; i < argc; i++) {
		arg = argv[i];
		if (strcmp(arg,"-a") == 0 || strcmp(arg,"--admin") == 0)
			clex_data.admin = 1;
		else if (strcmp(arg,"--help") == 0)
			help = 1;
		else if (strcmp(arg,"--version") == 0)
			version = 1;
		else
			err_exit("Unrecognized option '%s'\n"
			  "    try '%s --help' for more information",
			  arg,base_name(argv[0]));
	}
	if (version)
		puts(
		  "\nCLEX File Manager " VERSION "\n"
#ifdef _POSIX_JOB_CONTROL
		  "  compiled with _POSIX_JOB_CONTROL\n"
#endif
		  "Copyright (C) 2001-2003 Vlado Potisk <vlado_potisk@clex.sk>"
		  "\n\n"
		  "This is free software distributed without any warranty.\n"
		  "See the GNU General Public License for more details.\n"
		  "\n"
		  "Project homepage is http://www.clex.sk");
	if (help)
		printf(
		  "\nUsage: %s [OPTION]\n\n"
		  "  -a, --admin      admin mode (system-wide configuration)\n"
		  "      --version    display program version and exit\n"
		  "      --help       display this help and exit\n",
		  base_name(argv[0]));
	if (help || version)
		exit(0);	/* no cleanup necessary at this stage */

	/* real start */
	puts("\n\n\n\nStarting CLEX " VERSION);

#ifdef HAVE_PUTENV
	putenv("CLEX=" VERSION);	/* let's have $CLEX (just so) */
#endif

	/* initialize program data */
	clex_data.pid = getpid();
	clex_data.umask = umask(0777);
	umask(clex_data.umask);
	clex_data.isroot = geteuid() == 0;  /* 0 or 1 */;

	US_INIT(line_tmp.line);	/* belongs to nowhere, initialize it here */
	US_INIT(line_tmp.save_line);

	signal_initialize();	/* set signal dispositions */
	userdata_initialize();	/* must be done before config */
	config_initialize();	/* read configuration */

	/* initialize the rest, the order is no longer important */
	help_initialize();
	if (!clex_data.admin) {
		/* all these features are not used in admin mode */
		completion_initialize();
		dir_initialize();
		exec_initialize();
		files_initialize();
		hist_initialize();
		list_initialize();
		select_initialize();
		xterm_title_initialize();
	}

	/* start CURSES screen handling */
	tty_initialize();
	curses_initialize();

	/* let's go ! */
	cx_version();
	control_loop(clex_data.admin ? MODE_CFG : MODE_FILE);

	/* NOTREACHED */
	return 0;
}
